@@ -15,6 +15,38 @@ T = TypeVar("T")
 U = TypeVar("U")
 
 
+class WeightedValue(Generic[T]):
+    """Represents a value with an associated weight for weighted selection."""
+
+    def __init__(self, value: T, weight: float):
+        self.value = value
+        self.weight = weight
+
+
+class Weighted(Generic[T]):
+    """Wraps a Generator with an associated weight for weighted selection."""
+
+    def __init__(self, generator: "Generator[T]", weight: float):
+        self.generator = generator
+        self.weight = weight
+
+    def generate(self, rng: "Random") -> "Shrinkable[T]":
+        """Generate a value using the wrapped generator."""
+        return self.generator.generate(rng)
+
+    def map(self, transformer: Callable[[T], U]) -> "Generator[U]":
+        """Apply a transformation to the wrapped generator."""
+        return self.generator.map(transformer)
+
+    def filter(self, predicate: Callable[[T], bool]) -> "Generator[T]":
+        """Filter the wrapped generator."""
+        return self.generator.filter(predicate)
+
+    def flat_map(self, gen_factory: Callable[[T], "Generator[U]"]) -> "Generator[U]":
+        """Apply flat_map to the wrapped generator."""
+        return self.generator.flat_map(gen_factory)
+
+
 class Random(Protocol):
     """Protocol for random number generators."""
 
@@ -31,6 +63,102 @@ class Random(Protocol):
         ...
 
 
+def is_weighted_value(element: Any) -> bool:
+    """Type check to determine if an element is weighted."""
+    return isinstance(element, WeightedValue)
+
+
+def is_weighted_generator(gen: Any) -> bool:
+    """Type check to determine if a generator is weighted."""
+    return isinstance(gen, Weighted)
+
+
+def normalize_weighted_values(values: List[Any]) -> List[WeightedValue]:
+    """Normalize weights so they sum to 1.0, handling mixed weighted/unweighted."""
+    if not values:
+        raise ValueError("At least one value must be provided")
+
+    sum_weight = 0.0
+    num_unassigned = 0
+
+    # First pass: collect weighted values and count unweighted ones
+    weighted_values = []
+    for raw_or_weighted in values:
+        if is_weighted_value(raw_or_weighted):
+            weighted = raw_or_weighted
+            sum_weight += weighted.weight
+            weighted_values.append(weighted)
+        else:
+            num_unassigned += 1
+            # Temporarily assign 0 weight to unweighted values
+            weighted_values.append(WeightedValue(raw_or_weighted, 0.0))
+
+    # Validate the sum of explicitly assigned weights
+    if sum_weight < 0.0 or sum_weight > 1.0:
+        raise ValueError(
+            "invalid weights: sum must be between 0.0 (exclusive) and 1.0 (inclusive)"
+        )
+
+    # Distribute remaining probability mass among unweighted values
+    if num_unassigned > 0:
+        rest = 1.0 - sum_weight
+        if rest <= 0.0:
+            raise ValueError(
+                "invalid weights: rest of weights must be greater than 0.0"
+            )
+
+        per_unassigned = rest / num_unassigned
+        weighted_values = [
+            WeightedValue(wv.value, per_unassigned) if wv.weight == 0.0 else wv
+            for wv in weighted_values
+        ]
+
+    return weighted_values
+
+
+def normalize_weighted_generators(generators: List[Any]) -> List[Weighted]:
+    """Normalize weights so they sum to 1.0, handling mixed weighted/unweighted."""
+    if not generators:
+        raise ValueError("At least one generator must be provided")
+
+    sum_weight = 0.0
+    num_unassigned = 0
+
+    # First pass: collect weighted generators and count unweighted ones
+    weighted_generators = []
+    for raw_or_weighted in generators:
+        if is_weighted_generator(raw_or_weighted):
+            weighted = raw_or_weighted
+            sum_weight += weighted.weight
+            weighted_generators.append(weighted)
+        else:
+            num_unassigned += 1
+            # Temporarily assign 0 weight to unweighted generators
+            weighted_generators.append(Weighted(raw_or_weighted, 0.0))
+
+    # Validate the sum of explicitly assigned weights
+    if sum_weight < 0.0 or sum_weight > 1.0:
+        raise ValueError(
+            "invalid weights: sum must be between 0.0 (exclusive) and 1.0 (inclusive)"
+        )
+
+    # Distribute remaining probability mass among unweighted generators
+    if num_unassigned > 0:
+        rest = 1.0 - sum_weight
+        if rest <= 0.0:
+            raise ValueError(
+                "invalid weights: rest of weights must be greater than 0.0"
+            )
+
+        per_unassigned = rest / num_unassigned
+        weighted_generators = [
+            Weighted(wg.generator, per_unassigned) if wg.weight == 0.0 else wg
+            for wg in weighted_generators
+        ]
+
+    return weighted_generators
+
+
 class Generator(ABC, Generic[T]):
     """Abstract base class for generators."""
 
@@ -90,12 +218,31 @@ class FilteredGenerator(Generator[T]):
         for _ in range(self.max_attempts):
             shrinkable = self.generator.generate(rng)
             if self.predicate(shrinkable.value):
-                return shrinkable
+                # Create a new Shrinkable with filtered shrinking candidates
+                filtered_shrinks = self._filter_shrinks(shrinkable)
+                return Shrinkable(shrinkable.value, lambda: filtered_shrinks)
         raise ValueError(
             f"Could not generate value satisfying predicate after "
             f"{self.max_attempts} attempts"
         )
 
+    def _filter_shrinks(self, shrinkable: Shrinkable[T]):
+        """Filter shrinking candidates by predicate."""
+        from python_proptest.core.stream import Stream
+
+        def filtered_stream():
+            original_stream = shrinkable.shrinks()
+            filtered_candidates = []
+
+            # Get all candidates from the original stream
+            for candidate in original_stream:
+                if self.predicate(candidate.value):
+                    filtered_candidates.append(candidate)
+
+            return Stream.many(filtered_candidates)
+
+        return filtered_stream()
+
 
 class FlatMappedGenerator(Generator[U]):
     """Generator that generates a value, then uses it to generate another value."""
@@ -146,6 +293,13 @@ class Gen:
         """Generate random lists of elements from the given generator."""
         return ListGenerator(element_generator, min_length, max_length)
 
+    @staticmethod
+    def unique_list(
+        element_generator: "Generator", min_length: int = 0, max_length: int = 10
+    ) -> "UniqueListGenerator":
+        """Generate random lists with unique elements from the given generator."""
+        return UniqueListGenerator(element_generator, min_length, max_length)
+
     @staticmethod
     def dict(
         key_generator: "Generator",
@@ -158,12 +312,12 @@ class Gen:
 
     @staticmethod
     def one_of(*generators):
-        """Randomly choose from multiple generators."""
+        """Randomly choose from multiple generators with optional weights."""
         return OneOfGenerator(list(generators))
 
     @staticmethod
     def element_of(*values):
-        """Randomly choose from multiple values."""
+        """Randomly choose from multiple values with optional weights."""
         if not values:
             raise ValueError("At least one value must be provided")
         return ElementOfGenerator(list(values))
@@ -174,9 +328,14 @@ class Gen:
         return JustGenerator(value)
 
     @staticmethod
-    def weighted_value(generator: "Generator", weight: float) -> "Generator":
-        """Create a weighted generator (for compatibility with stateful tests)."""
-        return generator
+    def weighted_gen(generator: "Generator", weight: float) -> "Weighted":
+        """Wraps a generator with a weight for Gen.one_of."""
+        return Weighted(generator, weight)
+
+    @staticmethod
+    def weighted_value(value: T, weight: float) -> "WeightedValue":
+        """Wraps a value with a weight for Gen.element_of."""
+        return WeightedValue(value, weight)
 
     @staticmethod
     def set(
@@ -192,11 +351,47 @@ class Gen:
         """Generate random Unicode strings with the specified constraints."""
         return UnicodeStringGenerator(min_length, max_length)
 
+    @staticmethod
+    def ascii_string(min_length: int = 0, max_length: int = 20) -> "StringGenerator":
+        """Generate random ASCII strings (characters 0-127)."""
+        return StringGenerator(min_length, max_length, "ascii")
+
+    @staticmethod
+    def printable_ascii_string(
+        min_length: int = 0, max_length: int = 20
+    ) -> "StringGenerator":
+        """Generate random printable ASCII strings (characters 32-126)."""
+        return StringGenerator(min_length, max_length, "printable_ascii")
+
+    @staticmethod
+    def ascii_char() -> "IntGenerator":
+        """Generate single ASCII character codes (0-127)."""
+        return IntGenerator(0, 127)
+
+    @staticmethod
+    def unicode_char() -> "UnicodeCharGenerator":
+        """Generate single Unicode character codes (avoiding surrogate pairs)."""
+        return UnicodeCharGenerator()
+
+    @staticmethod
+    def printable_ascii_char() -> "IntGenerator":
+        """Generate single printable ASCII character codes (32-126)."""
+        return IntGenerator(32, 126)
+
     @staticmethod
     def interval(min_value: int, max_value: int) -> "IntGenerator":
         """Generate random integers in the specified range (inclusive)."""
         return IntGenerator(min_value, max_value)
 
+    @staticmethod
+    def in_range(min_value: int, max_value: int) -> "IntGenerator":
+        """Generate random integers in range [min_value, max_value) (exclusive)."""
+        min_val: int = min_value
+        max_val: int = max_value
+        if min_val >= max_val:
+            raise ValueError(f"invalid range: min ({min_val}) >= max ({max_val})")
+        return IntGenerator(min_val, max_val - 1)
+
     @staticmethod
     def integers(min_value: int, max_value: int) -> "IntGenerator":
         """Alias for interval for compatibility."""
@@ -268,20 +463,110 @@ class IntGenerator(Generator[int]):
         """Generate shrinking candidates for an integer."""
         shrinks = []
 
-        # Shrink towards zero
-        if value > 0:
-            shrinks.append(Shrinkable(0))
-            if value > 1:
-                shrinks.append(Shrinkable(1))
-        elif value < 0:
-            shrinks.append(Shrinkable(0))
-            if value < -1:
-                shrinks.append(Shrinkable(-1))
+        # Shrink towards min_value (or zero if min_value <= 0)
+        target = max(self.min_value, 0)
+        if value > target:
+            if target >= self.min_value and target <= self.max_value:
+                shrinks.append(Shrinkable(target))
+            if (
+                value > target + 1
+                and target + 1 >= self.min_value
+                and target + 1 <= self.max_value
+            ):
+                shrinks.append(Shrinkable(target + 1))
+        elif value < target:
+            if target <= self.max_value and target >= self.min_value:
+                shrinks.append(Shrinkable(target))
+            if (
+                value < target - 1
+                and target - 1 <= self.max_value
+                and target - 1 >= self.min_value
+            ):
+                shrinks.append(Shrinkable(target - 1))
+
+        # Binary search shrinking towards min_value
+        if value != self.min_value:
+            # Try shrinking towards min_value
+            if value > self.min_value:
+                mid = (value + self.min_value) // 2
+                if mid >= self.min_value and mid <= self.max_value and mid != value:
+                    shrinks.append(Shrinkable(mid))
+
+            # Try shrinking towards max_value (for negative values)
+            if value < self.max_value:
+                mid = (value + self.max_value) // 2
+                if mid >= self.min_value and mid <= self.max_value and mid != value:
+                    shrinks.append(Shrinkable(mid))
+
+        return shrinks
+
+
+class UniqueListGenerator(Generator[List[T]]):
+    """Generator for lists with unique elements."""
+
+    def __init__(
+        self, element_generator: "Generator[T]", min_length: int, max_length: int
+    ):
+        self.element_generator = element_generator
+        self.min_length = min_length
+        self.max_length = max_length
+
+    def generate(self, rng: Random) -> Shrinkable[List[T]]:
+        """Generate a list with unique elements."""
+        # Use set generator and convert to list
+        set_gen = SetGenerator(self.element_generator, self.min_length, self.max_length)
+        set_shrinkable = set_gen.generate(rng)
+
+        # Convert set to sorted list
+        unique_list = list(set_shrinkable.value)
+        unique_list.sort()
+
+        # Generate shrinks by converting set shrinks to list shrinks
+        shrinks = []
+        for set_shrink in set_shrinkable.shrinks():
+            shrink_list = list(set_shrink.value)
+            shrink_list.sort()
+            shrinks.append(Shrinkable(shrink_list))
+
+        from python_proptest.core.stream import Stream
+
+        return Shrinkable(unique_list, lambda: Stream.many(shrinks))
+
+
+class UnicodeCharGenerator(Generator[int]):
+    """Generator for Unicode character codes avoiding surrogate pairs."""
+
+    def generate(self, rng: Random) -> Shrinkable[int]:
+        """Generate a Unicode character code avoiding surrogate pairs."""
+        # Generate a random number in the range [1, 0xD7FF + (0x10FFFF - 0xE000 + 1)]
+        # Then map it to avoid surrogate pairs (U+D800 to U+DFFF)
+        max_range = 0xD7FF + (0x10FFFF - 0xE000 + 1)
+        code = rng.randint(1, max_range)
+
+        # Skip surrogate pair range D800-DFFF
+        if code >= 0xD800:
+            code += 0xE000 - 0xD800
+
+        shrinks = self._generate_shrinks(code)
+        from python_proptest.core.stream import Stream
+
+        return Shrinkable(code, lambda: Stream.many(shrinks))
+
+    def _generate_shrinks(self, value: int) -> List[Shrinkable[int]]:
+        """Generate shrinking candidates for a Unicode character code."""
+        shrinks = []
+
+        # Shrink towards 1 (minimum valid Unicode)
+        if value > 1:
+            shrinks.append(Shrinkable(1))
+            if value > 2:
+                shrinks.append(Shrinkable(2))
 
         # Binary search shrinking
-        if abs(value) > 1:
-            shrinks.append(Shrinkable(value // 2))
-            shrinks.append(Shrinkable(-value // 2))
+        if value > 1:
+            mid = (value + 1) // 2
+            if mid != value:
+                shrinks.append(Shrinkable(mid))
 
         return shrinks
 
@@ -292,7 +577,19 @@ class StringGenerator(Generator[str]):
     def __init__(self, min_length: int, max_length: int, charset: str):
         self.min_length = min_length
         self.max_length = max_length
-        self.charset = charset
+        self.charset = self._get_charset(charset)
+
+    def _get_charset(self, charset: str) -> str:
+        """Convert charset specification to actual character set."""
+        if charset == "ascii":
+            # ASCII characters 0-127
+            return "".join(chr(i) for i in range(128))
+        elif charset == "printable_ascii":
+            # Printable ASCII characters 32-126
+            return "".join(chr(i) for i in range(32, 127))
+        else:
+            # Use the provided charset as-is
+            return charset
 
     def generate(self, rng: Random) -> Shrinkable[str]:
         length = rng.randint(self.min_length, self.max_length)
@@ -483,33 +780,46 @@ class DictGenerator(Generator[Dict[T, U]]):
 
 
 class OneOfGenerator(Generator[T]):
-    """Generator that randomly chooses from multiple generators."""
+    """Generator that randomly chooses from multiple generators with weights."""
 
-    def __init__(self, generators: List[Generator[T]]):
+    def __init__(self, generators: List[Any]):
         if not generators:
             raise ValueError("At least one generator must be provided")
-        self.generators = generators
+        self.weighted_generators = normalize_weighted_generators(generators)
 
     def generate(self, rng: Random) -> Shrinkable[T]:
-        generator = rng.choice(self.generators)
-        return generator.generate(rng)
+        # Selection loop: repeatedly pick a generator index and check against its weight
+        while True:
+            dice = rng.randint(0, len(self.weighted_generators) - 1)
+            weighted_gen = self.weighted_generators[dice]
+            if rng.random() < weighted_gen.weight:
+                return weighted_gen.generate(rng)
 
 
 class ElementOfGenerator(Generator[T]):
-    """Generator that randomly chooses from multiple values."""
+    """Generator that randomly chooses from multiple values with optional weights."""
 
-    def __init__(self, values: List[T]):
+    def __init__(self, values: List[Any]):
         if not values:
             raise ValueError("At least one value must be provided")
-        self.values = values
+        self.weighted_values = normalize_weighted_values(values)
 
     def generate(self, rng: Random) -> Shrinkable[T]:
-        value = rng.choice(self.values)
-        # Generate shrinks by trying other values
-        shrinks = [Shrinkable(v) for v in self.values if v != value]
-        from python_proptest.core.stream import Stream
+        # Selection loop: repeatedly pick a value index and check against its weight
+        while True:
+            dice = rng.randint(0, len(self.weighted_values) - 1)
+            weighted_value = self.weighted_values[dice]
+            if rng.random() < weighted_value.weight:
+                value = weighted_value.value
+                # Generate shrinks by trying other values
+                shrinks = [
+                    Shrinkable(wv.value)
+                    for wv in self.weighted_values
+                    if wv.value != value
+                ]
+                from python_proptest.core.stream import Stream
 
-        return Shrinkable(value, lambda: Stream.many(shrinks))
+                return Shrinkable(value, lambda: Stream.many(shrinks))
 
 
 class JustGenerator(Generator[T]):
